#!/usr/sbin/dtrace -Zs
/*
 * js_funcflow.d - snoop JavaScript function execution using DTrace.
 *                 Written for the Solaris Nevada DTrace JavaScript provider.
 *
 * 12-Sep-2006, ver 0.70
 *
 * USAGE: js_funcflow.d			# hit Ctrl-C to end
 *
 * This watches JavaScript function entries and returns, and indents child
 * function calls. For busy JavaScript, the output will be very verbose.
 *
 * FIELDS:
 *		C		CPU-id
 *		TIME		Time of function call
 *		FILE		Filename that this function belongs to
 *		FUNC		JavaScript function name
 *
 * WARNING: Watch the first column carefully, it prints the CPU-id. If it
 * changes, then it is very likely that the output has been shuffled.
 */
/*
 * Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
#pragma ident	"@(#)js_funcflow.d	1.1	06/09/15 SMI"

#pragma D option quiet
#pragma D option switchrate=10

dtrace:::BEGIN
{
	depth = 0;
	printf("%s %-20s  %-22s   -- %s\n", "C", "TIME", "FILE", "FUNC");
}

javascript*:::function-entry
{
	depth++;
	printf("%d %-20Y  %-22s %*s-> %s\n", cpu, walltimestamp, 
	    basename(copyinstr(arg0)), depth*2, "", copyinstr(arg2));
}

javascript*:::function-return
{
	printf("%d %-20Y  %-22s %*s<- %s\n", cpu, walltimestamp,
	    basename(copyinstr(arg0)), depth*2, "", copyinstr(arg2));
	depth--;
}
