#!/usr/sbin/dtrace -Zs
/*
 * js_functime.d - measure JavaScript function times using DTrace.
 *		   Written for the Solaris Nevada DTrace JavaScript provider.
 *
 * 18-Sep-2006, ver 0.55	# in development, check for updates
 *
 * USAGE: js_functime.d		# hit Ctrl-C to end
 *
 * FIELDS:
 *		FILE		Filename that contained the function
 *		FUNCTION	JavaScript function name
 *		COUNT		Number of calls during this sample
 *		AVG(us)		Average function elapsed time, us
 *		SUM(us)		Total function elapsed time, us
 *
 * This measures the elapsed times for function entries to returns, and
 * provides some aggregated statistics. There are some points to be aware
 * of (these may improve in future versions):
 *
 *	- the elapsed time for a function is overlapping; it includes the
 *	  elapsed times for child functions that it called.
 */
/*
 * Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
#pragma ident	"%Z%%M%	%I%	%E% SMI"

#pragma D option quiet

this string str;

dtrace:::BEGIN
{
	printf("Tracing... Hit Ctrl-C to end.\n");
	depth = 0;
}

javascript*:::function-entry
{
	self->depth++;
	self->start[copyinstr(arg2), self->depth] = timestamp;
}

javascript*:::function-return
/(this->str = copyinstr(arg2)) != NULL && self->start[this->str, self->depth]/
{
	this->file = basename(copyinstr(arg0));
	this->elapsed = timestamp - self->start[this->str, self->depth];
	@num[this->file, this->str] = count();
	@eavg[this->file, this->str] = avg(this->elapsed);
	@esum[this->file, this->str] = sum(this->elapsed);
	@edist[this->str] = quantize(this->elapsed);
	self->start[this->file, self->depth] = 0;
	self->depth--;
}

dtrace:::END
{
	normalize(@eavg, 1000);
	normalize(@esum, 1000);
	printf("ELAPSED TIME DISTRIBUTION,\n");
	printa(@edist);
	setopt("aggsortpos", "2");
	printf("%-33s %45s\n", "___ OVERLAP TIMES: ___",
	    "______ ELAPSED _____");
	printf("%-24s %-23s %6s %10s %12s\n", "FILE", "FUNCTION",
	    "COUNT", "AVG(us)", "SUM(us)");
	printa("%-24.24s %-23.23s %@6d %@10d %@12d\n", @num, @eavg, @esum);
}
