#!/usr/sbin/dtrace -Zs
/*
 * js_objcpu.d - measure JavaScript Object create on-CPU time using DTrace.
 *               Written for the Solaris Nevada DTrace JavaScript provider.
 *
 * 14-Sep-2006, ver 0.70
 *
 * USAGE: js_objcpu.d		# hit Ctrl-C to end
 *
 * This measures the on-CPU time consumed due to JavaScript Object creation.
 */
/*
 * Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
#pragma ident	"@(#)js_objcpu.d	1.1	06/09/15 SMI"

#pragma D option quiet

dtrace:::BEGIN
{
	printf("Tracing... Hit Ctrl-C to end.\n");
}

javascript*:::object-create-start
{
	self->vstart = vtimestamp;
}

javascript*:::object-create-done
/self->vstart/
{
	this->oncpu = vtimestamp - self->vstart;
	@dist["Object Creation on-CPU time (ns):"] = quantize(this->oncpu);
	@total["Total on-CPU time (us):"] = sum(this->oncpu);
	self->vstart = 0;
}

dtrace:::END
{
	normalize(@total, 1000);
}
