#!/usr/sbin/dtrace -Zs
/*
 * js_objgc.d - measure JavaScript Object GC using DTrace.
 *              Written for the Solaris Nevada DTrace JavaScript provider.
 *
 * 14-Sep-2006, ver 0.80
 *
 * USAGE: js_objgc.d		# hit Ctrl-C to end
 *
 * FIELDS:
 *              FILE		Filename that contained the function
 *              CLASS		Class to which this new object belongs
 *              TOTAL		Object entropy (positive == uncollected)
 *
 * This script provides information on which objects are not being garbage
 * collected, an issue which causes the browser to steadily leak memory.
 * We trace object creation (+1) and destruction (-1), and provide a
 * summary each second of the running tally of the object class and
 * originating filename. If over the period of several minutes an object
 * type is still steadily increasing, then that would be of interest.
 * Be patient, depending on the rate of object creation, it can take over
 * ten minutes for garbage collect to kick in.
 *
 * NOTE: there are other Things that GC handles, other than Objects; extra
 * probes can be added to trace them, should the need arise.
 */
/*
 * Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
#pragma ident	"@(#)js_objgc.d	1.1	06/09/15 SMI"

#pragma D option quiet
#pragma D option dynvarsize=32m

dtrace:::BEGIN
{
	printf("Tracing... Hit Ctrl-C to end.\n");
}

javascript*:::object-create
/arg2/
{
	this->file = basename(copyinstr(arg0));
	@objs[this->file, copyinstr(arg1)] = sum(1);
	filename[arg2] = this->file;
}

javascript*:::object-finalize
/filename[arg2] == NULL/
{
	/* these are caused by dynvardrops. increase dynvarsize */
	@objs["<missed>", copyinstr(arg1)] = sum(-1);
}

javascript*:::object-finalize
/filename[arg2] != NULL/
{
	@objs[filename[arg2], copyinstr(arg1)] = sum(-1);
	filename[arg2] = 0;
}

profile:::tick-1sec,
dtrace:::END
{
	printf("\n %-24s %-36s %8s\n", "FILE", "CLASS", "TOTAL");
	printa(" %-24.24s %-36s %@8d\n", @objs);
}
