/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * vim: set ts=8 sw=4 et tw=80:
 */
#pragma ident   "@(#)jsdtracef.c 1.1     06/09/15 SMI"

#include "jsapi.h"
#include "jsutil.h"
#include "jsatom.h"
#include "jsfun.h"
#include "jsobj.h"
#include "jsstr.h"
#include "jscntxt.h"
#include "jsdbgapi.h"
#include "jsscript.h"
#include "jsinterp.h"
#include "jsdtrace.h"
#include "jsdtracef.h"
#include <sys/types.h>

static char *dempty = "<null>";

/*
 * These functions call the DTrace macros for the JavaScript USDT probes.
 * Originally this code was inlined in the JavaScript code; however since
 * a number of operations are called (and more are planned, for stack traces),
 * these have been placed into functions to reduce any negative compiler 
 * optimization effect that the addition of a number of usually unused
 * lines of code would cause.
 */

void
jsdtrace_function_entry(JSStackFrame *fp, JSFunction *fun)
{
    JSString *dstr;

    dstr = fun && fun->atom ? ATOM_TO_STRING(fun->atom) : NULL;
    JAVASCRIPT_FUNCTION_ENTRY(fp->script && fp->script->filename ?
        (char *)fp->script->filename : dempty,
        fun && fun->clasp && fun->clasp->name ?
        (char *)fun->clasp->name : dempty,
        dstr ? (char *)JS_GetStringBytes(dstr) : dempty);
    return;
}

void
jsdtrace_function_return(JSStackFrame *fp, JSFunction *fun)
{
    JSString *dstr;

    dstr = fun && fun->atom ? ATOM_TO_STRING(fun->atom) : NULL;
    JAVASCRIPT_FUNCTION_RETURN(fp->script && fp->script->filename ?
        (char *)fp->script->filename : dempty,
        fun && fun->clasp && fun->clasp->name ?
        (char *)fun->clasp->name : dempty,
        dstr ? (char *)JS_GetStringBytes(dstr) : dempty);
    return;
}

void
jsdtrace_object_create_start(JSStackFrame *fp, JSClass *clasp)
{
    JAVASCRIPT_OBJECT_CREATE_START(
        fp && fp->script && fp->script->filename ?
        (char *)fp->script->filename : dempty,
        clasp && clasp->name ? (char *)clasp->name : dempty);
    return;
}

void
jsdtrace_object_create_done(JSStackFrame *fp, JSClass *clasp)
{
    JAVASCRIPT_OBJECT_CREATE_DONE(
        fp && fp->script && fp->script->filename ?
        (char *)fp->script->filename : dempty,
        clasp && clasp->name ? (char *)clasp->name : dempty);
    return;
}

void
jsdtrace_object_create(JSContext *cx, JSClass *clasp, JSObject *obj)
{
    JAVASCRIPT_OBJECT_CREATE(
        cx->fp && cx->fp->script && cx->fp->script->filename ?
        (char *)cx->fp->script->filename : dempty,
        clasp && clasp->name ? (char *)clasp->name : dempty, (uintptr_t)obj,
        cx->fp && cx->fp->script && cx->fp->pc ?
        JS_PCToLineNumber(cx, cx->fp->script, cx->fp->pc) : -1);
    return;
}

void
jsdtrace_object_finalize(JSObject *obj)
{
    JSClass *clasp;

    clasp = obj ? LOCKED_OBJ_GET_CLASS(obj) : NULL;

    /* the first arg is NULL - reserved for future use (filename?) */
    JAVASCRIPT_OBJECT_FINALIZE((char *)NULL, 
        clasp && clasp->name ? (char *)clasp->name : dempty, (uintptr_t)obj);
    return;
}

void
jsdtrace_execute_start(JSScript *script)
{
    JAVASCRIPT_EXECUTE_START(
        script && script->filename ?  (char *)script->filename : dempty,
        script ? script->lineno : -1);
    return;
}

void
jsdtrace_execute_done(JSScript *script)
{
    JAVASCRIPT_EXECUTE_DONE(
        script && script->filename ?  (char *)script->filename : dempty,
        script ? script->lineno : -1);
    return;
}
